"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _safeContract, _ethers, _ownerManager, _moduleManager, _provider, _signer;
Object.defineProperty(exports, "__esModule", { value: true });
const ethers_1 = require("ethers");
const SafeAbiV1_2_0_json_1 = __importDefault(require("./abis/SafeAbiV1-2-0.json"));
const moduleManager_1 = __importDefault(require("./managers/moduleManager"));
const ownerManager_1 = __importDefault(require("./managers/ownerManager"));
const utils_1 = require("./utils");
const signatures_1 = require("./utils/signatures");
const SafeSignature_1 = require("./utils/signatures/SafeSignature");
const gas_1 = require("./utils/transactions/gas");
const SafeTransaction_1 = __importDefault(require("./utils/transactions/SafeTransaction"));
const utils_2 = require("./utils/transactions/utils");
class EthersSafe {
    constructor() {
        _safeContract.set(this, void 0);
        _ethers.set(this, void 0);
        _ownerManager.set(this, void 0);
        _moduleManager.set(this, void 0);
        _provider.set(this, void 0);
        _signer.set(this, void 0);
    }
    /**
     * Creates an instance of the Safe Core SDK.
     *
     * @param ethers - Ethers v5 library
     * @param safeAddress - The address of the Safe account to use
     * @param providerOrSigner - Ethers provider or signer. If this parameter is not passed, Ethers defaultProvider will be used.
     * @returns The Safe Core SDK instance
     */
    static async create(ethers, safeAddress, providerOrSigner) {
        const safeSdk = new EthersSafe();
        await safeSdk.init(ethers, safeAddress, providerOrSigner);
        return safeSdk;
    }
    /**
     * Initializes the Safe Core SDK instance.
     *
     * @param ethers - Ethers v5 library
     * @param safeAddress - The address of the Safe account to use
     * @param providerOrSigner - Ethers provider or signer. If this parameter is not passed, Ethers defaultProvider will be used.
     * @throws "Signer must be connected to a provider"
     * @throws "Safe contract is not deployed in the current network"
     */
    async init(ethers, safeAddress, providerOrSigner) {
        const currentProviderOrSigner = providerOrSigner || ethers.getDefaultProvider();
        if (ethers_1.Signer.isSigner(currentProviderOrSigner)) {
            if (!currentProviderOrSigner.provider) {
                throw new Error('Signer must be connected to a provider');
            }
            __classPrivateFieldSet(this, _provider, currentProviderOrSigner.provider);
            __classPrivateFieldSet(this, _signer, currentProviderOrSigner);
        }
        else {
            __classPrivateFieldSet(this, _provider, currentProviderOrSigner);
            __classPrivateFieldSet(this, _signer, undefined);
        }
        const safeContractCode = await __classPrivateFieldGet(this, _provider).getCode(safeAddress);
        if (safeContractCode === '0x') {
            throw new Error('Safe contract is not deployed in the current network');
        }
        __classPrivateFieldSet(this, _ethers, ethers);
        __classPrivateFieldSet(this, _safeContract, new (__classPrivateFieldGet(this, _ethers).Contract)(safeAddress, SafeAbiV1_2_0_json_1.default, currentProviderOrSigner));
        __classPrivateFieldSet(this, _ownerManager, new ownerManager_1.default(__classPrivateFieldGet(this, _ethers), __classPrivateFieldGet(this, _safeContract)));
        __classPrivateFieldSet(this, _moduleManager, new moduleManager_1.default(__classPrivateFieldGet(this, _ethers), __classPrivateFieldGet(this, _safeContract)));
    }
    /**
     * Returns a new instance of the Safe Core SDK connecting the providerOrSigner and the safeAddress.
     *
     * @param providerOrSigner - Ethers provider or signer
     * @param safeAddress - The address of the Safe account to use
     */
    async connect(providerOrSigner, safeAddress) {
        return await EthersSafe.create(__classPrivateFieldGet(this, _ethers), safeAddress || __classPrivateFieldGet(this, _safeContract).address, providerOrSigner);
    }
    /**
     * Returns the connected provider.
     *
     * @returns The connected provider
     */
    getProvider() {
        return __classPrivateFieldGet(this, _provider);
    }
    /**
     * Returns the connected signer.
     *
     * @returns The connected signer
     */
    getSigner() {
        return __classPrivateFieldGet(this, _signer);
    }
    /**
     * Returns the address of the current Safe Proxy contract.
     *
     * @returns The address of the Safe Proxy contract
     */
    getAddress() {
        return __classPrivateFieldGet(this, _safeContract).address;
    }
    /**
     * Returns the Safe Master Copy contract version.
     *
     * @returns The Safe Master Copy contract version
     */
    async getContractVersion() {
        return __classPrivateFieldGet(this, _safeContract).VERSION();
    }
    /**
     * Returns the list of Safe owner accounts.
     *
     * @returns The list of owners
     */
    async getOwners() {
        return __classPrivateFieldGet(this, _ownerManager).getOwners();
    }
    /**
     * Returns the Safe nonce.
     *
     * @returns The Safe nonce
     */
    async getNonce() {
        return (await __classPrivateFieldGet(this, _safeContract).nonce()).toNumber();
    }
    /**
     * Returns the Safe threshold.
     *
     * @returns The Safe threshold
     */
    async getThreshold() {
        return __classPrivateFieldGet(this, _ownerManager).getThreshold();
    }
    /**
     * Returns the chainId of the connected network.
     *
     * @returns The chainId of the connected network
     */
    async getChainId() {
        return (await __classPrivateFieldGet(this, _provider).getNetwork()).chainId;
    }
    /**
     * Returns the ETH balance of the Safe.
     *
     * @returns The ETH balance of the Safe
     */
    async getBalance() {
        return ethers_1.BigNumber.from(await __classPrivateFieldGet(this, _provider).getBalance(this.getAddress()));
    }
    /**
     * Returns the list of addresses of all the enabled Safe modules.
     *
     * @returns The list of addresses of all the enabled Safe modules
     */
    async getModules() {
        return __classPrivateFieldGet(this, _moduleManager).getModules();
    }
    /**
     * Checks if a specific Safe module is enabled for the current Safe.
     *
     * @param moduleAddress - The desired module address
     * @returns TRUE if the module is enabled
     */
    async isModuleEnabled(moduleAddress) {
        return __classPrivateFieldGet(this, _moduleManager).isModuleEnabled(moduleAddress);
    }
    /**
     * Checks if a specific address is an owner of the current Safe.
     *
     * @param ownerAddress - The account address
     * @returns TRUE if the account is an owner
     */
    async isOwner(ownerAddress) {
        return __classPrivateFieldGet(this, _ownerManager).isOwner(ownerAddress);
    }
    /**
     * Returns a Safe transaction ready to be signed by the owners.
     *
     * @param safeTransaction - The minimum required data to create the transaction
     * @returns The Safe transaction
     */
    async createTransaction(tx) {
        const safeTransaction = await utils_2.standardizeSafeTransaction(__classPrivateFieldGet(this, _safeContract), tx);
        return new SafeTransaction_1.default(safeTransaction);
    }
    /**
     * Returns the transaction hash of a Safe transaction.
     *
     * @param safeTransaction - The Safe transaction
     * @returns The transaction hash of the Safe transaction
     */
    async getTransactionHash(safeTransaction) {
        const safeTransactionData = safeTransaction.data;
        const txHash = await __classPrivateFieldGet(this, _safeContract).getTransactionHash(safeTransactionData.to, safeTransactionData.value, safeTransactionData.data, safeTransactionData.operation, safeTransactionData.safeTxGas, safeTransactionData.baseGas, safeTransactionData.gasPrice, safeTransactionData.gasToken, safeTransactionData.refundReceiver, safeTransactionData.nonce);
        return txHash;
    }
    /**
     * Signs a hash using the current signer account.
     *
     * @param hash - The hash to sign
     * @returns The Safe signature
     * @throws "No signer provided"
     * @throws "Transactions can only be signed by Safe owners"
     */
    async signTransactionHash(hash) {
        if (!__classPrivateFieldGet(this, _signer)) {
            throw new Error('No signer provided');
        }
        const owners = await this.getOwners();
        const signerAddress = await __classPrivateFieldGet(this, _signer).getAddress();
        const addressIsOwner = owners.find((owner) => __classPrivateFieldGet(this, _signer) && utils_1.sameString(owner, signerAddress));
        if (!addressIsOwner) {
            throw new Error('Transactions can only be signed by Safe owners');
        }
        const messageArray = __classPrivateFieldGet(this, _ethers).utils.arrayify(hash);
        const signature = await __classPrivateFieldGet(this, _signer).signMessage(messageArray);
        return new SafeSignature_1.EthSignSignature(signerAddress, signature);
    }
    /**
     * Adds the signature of the current signer to the Safe transaction object.
     *
     * @param safeTransaction - The Safe transaction to be signed
     */
    async signTransaction(safeTransaction) {
        const txHash = await this.getTransactionHash(safeTransaction);
        const signature = await this.signTransactionHash(txHash);
        safeTransaction.addSignature(signature);
    }
    /**
     * Approves on-chain a hash using the current signer account.
     *
     * @param hash - The hash to approve
     * @param skipOnChainApproval - TRUE to avoid the Safe transaction to be approved on-chain
     * @returns The pre-validated signature
     * @throws "No signer provided"
     * @throws "Transaction hashes can only be approved by Safe owners"
     */
    async approveTransactionHash(hash) {
        if (!__classPrivateFieldGet(this, _signer)) {
            throw new Error('No signer provided');
        }
        const owners = await this.getOwners();
        const signerAddress = await __classPrivateFieldGet(this, _signer).getAddress();
        const addressIsOwner = owners.find((owner) => __classPrivateFieldGet(this, _signer) && utils_1.sameString(owner, signerAddress));
        if (!addressIsOwner) {
            throw new Error('Transaction hashes can only be approved by Safe owners');
        }
        return __classPrivateFieldGet(this, _safeContract).approveHash(hash);
    }
    /**
     * Returns a list of owners who have approved a specific Safe transaction.
     *
     * @param txHash - The Safe transaction hash
     * @returns The list of owners
     */
    async getOwnersWhoApprovedTx(txHash) {
        const owners = await this.getOwners();
        let ownersWhoApproved = [];
        for (const owner of owners) {
            const approved = await __classPrivateFieldGet(this, _safeContract).approvedHashes(owner, txHash);
            if (approved.gt(0)) {
                ownersWhoApproved.push(owner);
            }
        }
        return ownersWhoApproved;
    }
    /**
     * Returns the Safe transaction to enable a Safe module.
     *
     * @param moduleAddress - The desired module address
     * @returns The Safe transaction ready to be signed
     * @throws "Invalid module address provided"
     * @throws "Module provided is already enabled"
     */
    async getEnableModuleTx(moduleAddress) {
        const tx = await this.createTransaction({
            to: this.getAddress(),
            value: '0',
            data: await __classPrivateFieldGet(this, _moduleManager).encodeEnableModuleData(moduleAddress),
            nonce: (await __classPrivateFieldGet(this, _safeContract).nonce()).toNumber()
        });
        return tx;
    }
    /**
     * Returns the Safe transaction to disable a Safe module.
     *
     * @param moduleAddress - The desired module address
     * @param params - Contract method name and specific parameters
     * @returns The Safe transaction ready to be signed
     * @throws "Invalid module address provided"
     * @throws "Module provided is not enabled already"
     */
    async getDisableModuleTx(moduleAddress) {
        const tx = await this.createTransaction({
            to: this.getAddress(),
            value: '0',
            data: await __classPrivateFieldGet(this, _moduleManager).encodeDisableModuleData(moduleAddress),
            nonce: (await __classPrivateFieldGet(this, _safeContract).nonce()).toNumber()
        });
        return tx;
    }
    /**
     * Returns the Safe transaction to add an owner and optionally change the threshold.
     *
     * @param ownerAddress - The address of the new owner
     * @param threshold - The new threshold
     * @returns The Safe transaction ready to be signed
     * @throws "Invalid owner address provided"
     * @throws "Address provided is already an owner"
     * @throws "Threshold needs to be greater than 0"
     * @throws "Threshold cannot exceed owner count"
     */
    async getAddOwnerTx(ownerAddress, threshold) {
        const tx = await this.createTransaction({
            to: this.getAddress(),
            value: '0',
            data: await __classPrivateFieldGet(this, _ownerManager).encodeAddOwnerWithThresholdData(ownerAddress, threshold),
            nonce: (await __classPrivateFieldGet(this, _safeContract).nonce()).toNumber()
        });
        return tx;
    }
    /**
     * Returns the Safe transaction to remove an owner and optionally change the threshold.
     *
     * @param ownerAddress - The address of the owner that will be removed
     * @param threshold - The new threshold
     * @returns The Safe transaction ready to be signed
     * @throws "Invalid owner address provided"
     * @throws "Address provided is not an owner"
     * @throws "Threshold needs to be greater than 0"
     * @throws "Threshold cannot exceed owner count"
     */
    async getRemoveOwnerTx(ownerAddress, threshold) {
        const tx = await this.createTransaction({
            to: this.getAddress(),
            value: '0',
            data: await __classPrivateFieldGet(this, _ownerManager).encodeRemoveOwnerData(ownerAddress, threshold),
            nonce: (await __classPrivateFieldGet(this, _safeContract).nonce()).toNumber()
        });
        return tx;
    }
    /**
     * Returns the Safe transaction to replace an owner of the Safe with a new one.
     *
     * @param oldOwnerAddress - The old owner address
     * @param newOwnerAddress - The new owner address
     * @returns The Safe transaction ready to be signed
     * @throws "Invalid new owner address provided"
     * @throws "Invalid old owner address provided"
     * @throws "New address provided is already an owner"
     * @throws "Old address provided is not an owner"
     */
    async getSwapOwnerTx(oldOwnerAddress, newOwnerAddress) {
        const tx = await this.createTransaction({
            to: this.getAddress(),
            value: '0',
            data: await __classPrivateFieldGet(this, _ownerManager).encodeSwapOwnerData(oldOwnerAddress, newOwnerAddress),
            nonce: (await __classPrivateFieldGet(this, _safeContract).nonce()).toNumber()
        });
        return tx;
    }
    /**
     * Returns the Safe transaction to change the threshold.
     *
     * @param threshold - The new threshold
     * @returns The Safe transaction ready to be signed
     * @throws "Threshold needs to be greater than 0"
     * @throws "Threshold cannot exceed owner count"
     */
    async getChangeThresholdTx(threshold) {
        const tx = await this.createTransaction({
            to: this.getAddress(),
            value: '0',
            data: await __classPrivateFieldGet(this, _ownerManager).encodeChangeThresholdData(threshold),
            nonce: (await __classPrivateFieldGet(this, _safeContract).nonce()).toNumber()
        });
        return tx;
    }
    /**
     * Executes a Safe transaction.
     *
     * @param safeTransaction - The Safe transaction to execute
     * @param options - Execution configuration options
     * @returns The Safe transaction response
     * @throws "No signer provided"
     * @throws "There are X signatures missing"
     */
    async executeTransaction(safeTransaction) {
        if (!__classPrivateFieldGet(this, _signer)) {
            throw new Error('No signer provided');
        }
        const txHash = await this.getTransactionHash(safeTransaction);
        const ownersWhoApprovedTx = await this.getOwnersWhoApprovedTx(txHash);
        for (const owner of ownersWhoApprovedTx) {
            safeTransaction.addSignature(signatures_1.generatePreValidatedSignature(owner));
        }
        const owners = await this.getOwners();
        const signerAddress = await __classPrivateFieldGet(this, _signer).getAddress();
        if (owners.includes(signerAddress)) {
            safeTransaction.addSignature(signatures_1.generatePreValidatedSignature(signerAddress));
        }
        const threshold = await this.getThreshold();
        if (threshold > safeTransaction.signatures.size) {
            const signaturesMissing = threshold - safeTransaction.signatures.size;
            throw new Error(`There ${signaturesMissing > 1 ? 'are' : 'is'} ${signaturesMissing} signature${signaturesMissing > 1 ? 's' : ''} missing`);
        }
        const gasLimit = await gas_1.estimateGasForTransactionExecution(__classPrivateFieldGet(this, _safeContract), await __classPrivateFieldGet(this, _signer).getAddress(), safeTransaction);
        const txResponse = await __classPrivateFieldGet(this, _safeContract).execTransaction(safeTransaction.data.to, safeTransaction.data.value, safeTransaction.data.data, safeTransaction.data.operation, safeTransaction.data.safeTxGas, safeTransaction.data.baseGas, safeTransaction.data.gasPrice, safeTransaction.data.gasToken, safeTransaction.data.refundReceiver, safeTransaction.encodedSignatures(), { gasLimit });
        return txResponse;
    }
}
_safeContract = new WeakMap(), _ethers = new WeakMap(), _ownerManager = new WeakMap(), _moduleManager = new WeakMap(), _provider = new WeakMap(), _signer = new WeakMap();
exports.default = EthersSafe;
//# sourceMappingURL=EthersSafe.js.map